/*
 * Craftsman Spy.
 * Copyright (C) 2005  Sbastien LECACHEUR
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package craftsman.spy;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.Ref;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

/**
 * The classe used to execute SQL stored procedures.
 * 
 * @author Sbastien LECACHEUR
 */
public class SpyCallableStatement extends SpyPreparedStatement implements CallableStatement {
	/**
	 * The real JDBC callable statement instance.
	 */
	private CallableStatement real = null;


	/**
	 * Indicates if the output parameters were already logged.
	 */
	private boolean outputParametersLogged = false;


	/**
	 * The list of all the output parameters type.
	 */
	private ArrayList outputParametersIndex = null;


	/**
	 * The list of all the output parameters name.
	 */
	private HashMap outputParametersName = null;


	/**
	 * Constructs a new Spy JDBC callable statement.
	 * @param c Connection The used connection.
	 * @param cstm CallableStatement The real JDBC callable statement.
	 * @param sql String The callable SQL.
	 */
	protected SpyCallableStatement ( Connection c, CallableStatement cstm, String sql) {
		super(c, cstm, sql);
		real = cstm;

		// Initialize capacity with the number of '?' in sql string
		outputParametersIndex = new ArrayList(getEstimatedParametersCount()+1);
		for (int i = 0; i < getEstimatedParametersCount()+1; i++) outputParametersIndex.add(i,null);
		outputParametersName = new HashMap(getEstimatedParametersCount()+1);
	}


	/**
	 * Registers the given output parameter.
	 * 
	 * @param paramIndex int The index position of the parameter.
	 * @param type int The type of the parameter.
	 */
	private void registerOutputParameter (int paramIndex, int type) {
		if ( outputParametersIndex.size()<=paramIndex) {
			outputParametersIndex.ensureCapacity(paramIndex+1);
			for ( int i = outputParametersIndex.size(); i < paramIndex; i++) outputParametersIndex.add(i,null);
				outputParametersIndex.add(paramIndex,new Integer(type));
		} else {
			outputParametersIndex.set(paramIndex,new Integer(type));
		}
	}


	/**
	 * Registers the given output parameter.
	 * 
	 * @param paramName String The name of the parameter.
	 * @param type int The type of the parameter.
	 */
	private void registerOutputParameter (String paramName, int type) {
		outputParametersName.put(paramName,new Integer(type));
	}


	/**
	 * @see SpyPreparedStatement#clearParameters()
	 */
	public void clearParameters() throws SQLException {
		outputParametersIndex.clear();
		outputParametersName.clear();
		super.clearParameters();
	}


	/**
	 * Logs all the output parameters.
	 */
	private void logOutputParameters() {
		//TODO add input parameter
		//TODO add output parameter registrered by name
		/*
		 * Cannot use {@link PreparedStatement#getParameterMetaData()} because some
		 * JDBC drivers don't implement this method.
		 */
		if ( outputParametersLogged==false) {
			StringBuffer displayableSql = new StringBuffer(preparedSql.length());


			if ( outputParametersIndex!=null && outputParametersName!=null) {
				int i = 1, limit = 0, base = 0;
				while ((limit = preparedSql.indexOf('?',limit)) != -1) {
					displayableSql.append(preparedSql.substring(base,limit));
					if ( outputParametersIndex.get(i)!=null) {
						switch ( ((Integer)outputParametersIndex.get(i)).intValue()) {
						case Types.ARRAY:
						case Types.BLOB:
						case Types.CHAR:
						case Types.CLOB:
						case Types.DATALINK:
						case Types.DATE:
						case Types.JAVA_OBJECT:
						case Types.LONGVARBINARY:
						case Types.LONGVARCHAR:
						case Types.REF:
						case Types.TIME:
						case Types.TIMESTAMP:
						case Types.VARBINARY:
						case Types.VARCHAR:			displayableSql.append("'");
													try {
														displayableSql.append(real.getString(i));
													} catch ( SQLException e) {
														if ( log.isErrorEnabled()) log.error(getId()+":"+"unable to retrieve output parameter",e);
													}
													displayableSql.append("'");
													break;
						default:					try {
														displayableSql.append(real.getString(i));
													} catch ( SQLException e) {
														if ( log.isErrorEnabled()) log.error(getId()+":"+"unable to retrieve output parameter",e);
													}
													break;	
						}
					} else {
						displayableSql.append("?");
					}
					i++;
					limit++;
					base = limit;
				}

				if (base < preparedSql.length()) {
					displayableSql.append(preparedSql.substring(base));
				}
			}

			log.info(getId()+":"+displayableSql.toString());
			outputParametersLogged = true;
		}
	}


	/**
	 * @see CallableStatement#wasNull()
	 */
	public boolean wasNull() throws SQLException {
		return real.wasNull();
	}


	/**
	 * @see CallableStatement#getByte(int)
	 */
	public byte getByte(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getByte(parameterIndex);
	}


	/**
	 * @see CallableStatement#getDouble(int)
	 */
	public double getDouble(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getDouble(parameterIndex);
	}


	/**
	 * @see CallableStatement#getFloat(int)
	 */
	public float getFloat(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getFloat(parameterIndex);
	}


	/**
	 * @see CallableStatement#getInt(int)
	 */
	public int getInt(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getInt(parameterIndex);
	}


	/**
	 * @see CallableStatement#getLong(int)
	 */
	public long getLong(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getLong(parameterIndex);
	}


	/**
	 * @see CallableStatement#getShort(int)
	 */
	public short getShort(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getShort(parameterIndex);
	}


	/**
	 * @see CallableStatement#getBoolean(int)
	 */
	public boolean getBoolean(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getBoolean(parameterIndex);
	}


	/**
	 * @see CallableStatement#getBytes(int)
	 */
	public byte[] getBytes(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getBytes(parameterIndex);
	}


	/**
	 * @see CallableStatement#registerOutParameter(int, int)
	 */
	public void registerOutParameter(int parameterIndex, int sqlType) throws SQLException {
		registerOutputParameter(parameterIndex,sqlType);
		real.registerOutParameter(parameterIndex,sqlType);
	}


	/**
	 * @see CallableStatement#registerOutParameter(int, int, int)
	 */
	public void registerOutParameter(int parameterIndex, int sqlType, int scale) throws SQLException {
		registerOutputParameter(parameterIndex,sqlType);
		real.registerOutParameter(parameterIndex,sqlType,scale);
	}


	/**
	 * @see CallableStatement#getObject(int)
	 */
	public Object getObject(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getObject(parameterIndex);
	}


	/**
	 * @see CallableStatement#getString(int)
	 */
	public String getString(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getString(parameterIndex);
	}


	/**
	 * @see CallableStatement#registerOutParameter(int, int, String)
	 */
	public void registerOutParameter(int paramIndex, int sqlType, String typeName) throws SQLException {
		registerOutputParameter(paramIndex,sqlType);
		real.registerOutParameter(paramIndex,sqlType,typeName);
	}


	/**
	 * @see CallableStatement#getByte(String)
	 */
	public byte getByte(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getByte(parameterName);
	}


	/**
	 * @see CallableStatement#getDouble(String)
	 */
	public double getDouble(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getDouble(parameterName);
	}


	/**
	 * @see CallableStatement#getFloat(String)
	 */
	public float getFloat(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getFloat(parameterName);
	}


	/**
	 * @see CallableStatement#getInt(String)
	 */
	public int getInt(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getInt(parameterName);
	}


	/**
	 * @see CallableStatement#getLong(String)
	 */
	public long getLong(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getLong(parameterName);
	}


	/**
	 * @see CallableStatement#getShort(String)
	 */
	public short getShort(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getShort(parameterName);
	}


	/**
	 * @see CallableStatement#getBoolean(String)
	 */
	public boolean getBoolean(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getBoolean(parameterName);
	}


	/**
	 * @see CallableStatement#getBytes(String)
	 */
	public byte[] getBytes(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getBytes(parameterName);
	}


	/**
	 * @see CallableStatement#setByte(String, byte)
	 */
	public void setByte(String parameterName, byte x) throws SQLException {
		real.setByte(parameterName,x);
	}


	/**
	 * @see CallableStatement#setDouble(String, double)
	 */
	public void setDouble(String parameterName, double x) throws SQLException {
		real.setDouble(parameterName,x);
	}


	/**
	 * @see CallableStatement#setFloat(String, float)
	 */
	public void setFloat(String parameterName, float x) throws SQLException {
		real.setFloat(parameterName,x);
	}


	/**
	 * @see CallableStatement#registerOutParameter(String, int)
	 */
	public void registerOutParameter(String parameterName, int sqlType) throws SQLException {
		registerOutputParameter(parameterName,sqlType);
		real.registerOutParameter(parameterName,sqlType);
	}


	/**
	 * @see CallableStatement#setInt(String, int)
	 */
	public void setInt(String parameterName, int x) throws SQLException {
		real.setInt(parameterName,x);
	}


	/**
	 * @see CallableStatement#setNull(String, int)
	 */
	public void setNull(String parameterName, int sqlType) throws SQLException {
		real.setNull(parameterName,sqlType);
	}


	/**
	 * @see CallableStatement#registerOutParameter(String, int, int)
	 */
	public void registerOutParameter(String parameterName, int sqlType, int scale) throws SQLException {
		registerOutputParameter(parameterName,sqlType);
		real.registerOutParameter(parameterName,sqlType,scale);
	}


	/**
	 * @see CallableStatement#setLong(String, long)
	 */
	public void setLong(String parameterName, long x) throws SQLException {
		real.setLong(parameterName,x);
	}


	/**
	 * @see CallableStatement#setShort(String, short)
	 */
	public void setShort(String parameterName, short x) throws SQLException {
		real.setShort(parameterName,x);
	}


	/**
	 * @see CallableStatement#setBoolean(String, boolean)
	 */
	public void setBoolean(String parameterName, boolean x) throws SQLException {
		real.setBoolean(parameterName,x);
	}


	/**
	 * @see CallableStatement#setBytes(String, byte[])
	 */
	public void setBytes(String parameterName, byte[] x) throws SQLException {
		real.setBytes(parameterName,x);
	}


	/**
	 * @see CallableStatement#getBigDecimal(int)
	 */
	public BigDecimal getBigDecimal(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getBigDecimal(parameterIndex);
	}


	/**
	 * @see CallableStatement#getBigDecimal(int, int)
	 */
	public BigDecimal getBigDecimal(int parameterIndex, int scale) throws SQLException {
		logOutputParameters();
		return real.getBigDecimal(parameterIndex,scale);
	}


	/**
	 * @see CallableStatement#getURL(int)
	 */
	public URL getURL(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getURL(parameterIndex);
	}


	/**
	 * @see CallableStatement#getArray(int)
	 */
	public Array getArray(int i) throws SQLException {
		logOutputParameters();
		return real.getArray(i);
	}


	/**
	 * @see CallableStatement#getBlob(int)
	 */
	public Blob getBlob(int i) throws SQLException {
		logOutputParameters();
		return real.getBlob(i);
	}


	/**
	 * @see CallableStatement#getClob(int)
	 */
	public Clob getClob(int i) throws SQLException {
		logOutputParameters();
		return real.getClob(i);
	}


	/**
	 * @see CallableStatement#getDate(int)
	 */
	public Date getDate(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getDate(parameterIndex);
	}


	/**
	 * @see CallableStatement#getRef(int)
	 */
	public Ref getRef(int i) throws SQLException {
		logOutputParameters();
		return real.getRef(i);
	}


	/**
	 * @see CallableStatement#getTime(int)
	 */
	public Time getTime(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getTime(parameterIndex);
	}


	/**
	 * @see CallableStatement#getTimestamp(int)
	 */
	public Timestamp getTimestamp(int parameterIndex) throws SQLException {
		logOutputParameters();
		return real.getTimestamp(parameterIndex);
	}


	/**
	 * @see CallableStatement#setAsciiStream(String, java.io.InputStream, int)
	 */
	public void setAsciiStream(String parameterName, InputStream x, int length) throws SQLException {
		real.setAsciiStream(parameterName,x,length);
	}


	/**
	 * @see CallableStatement#setBinaryStream(String, java.io.InputStream, int)
	 */
	public void setBinaryStream(String parameterName, InputStream x, int length) throws SQLException {
		real.setBinaryStream(parameterName,x,length);
	}


	/**
	 * @see CallableStatement#setCharacterStream(String, java.io.Reader, int)
	 */
	public void setCharacterStream(String parameterName, Reader reader, int length) throws SQLException {
		real.setCharacterStream(parameterName,reader,length);
	}


	/**
	 * @see CallableStatement#getObject(String)
	 */
	public Object getObject(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getObject(parameterName);
	}


	/**
	 * @see CallableStatement#setObject(String, Object)
	 */
	public void setObject(String parameterName, Object x) throws SQLException {
		real.setObject(parameterName,x);
	}


	/**
	 * @see CallableStatement#setObject(String, Object, int)
	 */
	public void setObject(String parameterName, Object x, int targetSqlType) throws SQLException {
		real.setObject(parameterName,x,targetSqlType);
	}


	/**
	 * @see CallableStatement#setObject(String, Object, int, int)
	 */
	public void setObject(String parameterName, Object x, int targetSqlType, int scale) throws SQLException {
		real.setObject(parameterName,x,targetSqlType,scale);
	}


	/**
	 * @see CallableStatement#getObject(int, Map)
	 */
	public Object getObject(int i, Map map) throws SQLException {
		logOutputParameters();
		return real.getObject(i,map);
	}


	/**
	 * @see CallableStatement#getString(String)
	 */
	public String getString(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getString(parameterName);
	}


	/**
	 * @see CallableStatement#registerOutParameter(String, int, String)
	 */
	public void registerOutParameter(String parameterName, int sqlType, String typeName) throws SQLException {
		registerOutputParameter(parameterName,sqlType);
		real.registerOutParameter(parameterName,sqlType,typeName);
	}


	/**
	 * @see CallableStatement#setNull(String, int, String)
	 */
	public void setNull(String parameterName, int sqlType, String typeName) throws SQLException {
		real.setNull(parameterName,sqlType,typeName);
	}


	/**
	 * @see CallableStatement#setString(String, String)
	 */
	public void setString(String parameterName, String x) throws SQLException {
		real.setString(parameterName,x);
	}


	/**
	 * @see CallableStatement#getBigDecimal(String)
	 */
	public BigDecimal getBigDecimal(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getBigDecimal(parameterName);
	}


	/**
	 * @see CallableStatement#setBigDecimal(String, BigDecimal)
	 */
	public void setBigDecimal(String parameterName, BigDecimal x) throws SQLException {
		real.setBigDecimal(parameterName,x);
	}


	/**
	 * @see CallableStatement#getURL(String)
	 */
	public URL getURL(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getURL(parameterName);
	}


	/**
	 * @see CallableStatement#setURL(String, java.net.URL)
	 */
	public void setURL(String parameterName, URL val) throws SQLException {
		real.setURL(parameterName,val);
	}


	/**
	 * @see CallableStatement#getArray(String)
	 */
	public Array getArray(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getArray(parameterName);
	}


	/**
	 * @see CallableStatement#getBlob(String)
	 */
	public Blob getBlob(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getBlob(parameterName);
	}


	/**
	 * @see CallableStatement#getClob(String)
	 */
	public Clob getClob(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getClob(parameterName);
	}


	/**
	 * @see CallableStatement#getDate(String)
	 */
	public Date getDate(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getDate(parameterName);
	}


	/**
	 * @see CallableStatement#setDate(String, Date)
	 */
	public void setDate(String parameterName, Date x) throws SQLException {
		real.setDate(parameterName,x);
	}


	/**
	 * @see CallableStatement#getDate(int, Calendar)
	 */
	public Date getDate(int parameterIndex, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getDate(parameterIndex,cal);
	}


	/**
	 * @see CallableStatement#getRef(String)
	 */
	public Ref getRef(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getRef(parameterName);
	}


	/**
	 * @see CallableStatement#getTime(String)
	 */
	public Time getTime(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getTime(parameterName);
	}


	/**
	 * @see CallableStatement#setTime(String, Time)
	 */
	public void setTime(String parameterName, Time x) throws SQLException {
		real.setTime(parameterName,x);
	}


	/**
	 * @see CallableStatement#getTime(int, Calendar)
	 */
	public Time getTime(int parameterIndex, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getTime(parameterIndex,cal);
	}


	/**
	 * @see CallableStatement#getTimestamp(String)
	 */
	public Timestamp getTimestamp(String parameterName) throws SQLException {
		logOutputParameters();
		return real.getTimestamp(parameterName);
	}


	/**
	 * @see CallableStatement#setTimestamp(String, Timestamp)
	 */
	public void setTimestamp(String parameterName, Timestamp x) throws SQLException {
		real.setTimestamp(parameterName,x);
	}


	/**
	 * @see CallableStatement#getTimestamp(int, Calendar)
	 */
	public Timestamp getTimestamp(int parameterIndex, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getTimestamp(parameterIndex,cal);
	}


	/**
	 * @see CallableStatement#getObject(String, Map)
	 */
	public Object getObject(String parameterName, Map map) throws SQLException {
		logOutputParameters();
		return real.getObject(parameterName,map);
	}


	/**
	 * @see CallableStatement#getDate(String, Calendar)
	 */
	public Date getDate(String parameterName, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getDate(parameterName,cal);
	}


	/**
	 * @see CallableStatement#getTime(String, Calendar)
	 */
	public Time getTime(String parameterName, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getTime(parameterName,cal);
	}


	/**
	 * @see CallableStatement#getTimestamp(String, Calendar)
	 */
	public Timestamp getTimestamp(String parameterName, Calendar cal) throws SQLException {
		logOutputParameters();
		return real.getTimestamp(parameterName,cal);
	}


	/**
	 * @see CallableStatement#setDate(String, Date, Calendar)
	 */
	public void setDate(String parameterName, Date x, Calendar cal) throws SQLException {
		real.setDate(parameterName,x,cal);
	}


	/**
	 * @see CallableStatement#setTime(String, Time, Calendar)
	 */
	public void setTime(String parameterName, Time x, Calendar cal) throws SQLException {
		real.setTime(parameterName,x,cal);
	}


	/**
	 * @see CallableStatement#setTimestamp(String, Timestamp, Calendar)
	 */
	public void setTimestamp(String parameterName, Timestamp x, Calendar cal) throws SQLException {
		real.setTimestamp(parameterName,x,cal);
	}
}
