/*
 * Craftsman Spy.
 * Copyright (C) 2005  Sbastien LECACHEUR
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package craftsman.spy;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.Properties;

/**
 * This classe is the implementation of the JDBC driver in order to
 * log any JDBC action.
 * 
 * The system property <code>spy.driver</code> MUST contains the real
 * JDBC driver full class name in order to use the Spy driver.
 * 
 * Another way is to use a specific JDBC url string like :
 * <code>jdbc:spy:<i>driver_full_class_name</i>:url</code>.
 * 
 * The two solutions can be used as the following examples :
 * <pre>
 * System.setProperty("spy.driver","vendor.database.Driver");// or with the -Dspy.driver=vendor.database.Driver JVM option
 * Class.forName("craftsman.spy.SpyDriver");
 * Connection c = DriverManager.getConnection("jdbc:database:mydata");
 * </pre>
 * 
 * <pre>
 * Class.forName("craftsman.spy.SpyDriver");
 * Connection c = DriverManager.getConnection("jdbc:spy:vendor.database.Driver:database:mydata");
 * </pre>
 * @author Sbastien LECACHEUR
 */
public class SpyDriver implements Driver {
	/**
	 * The real JDBC driver instance.
	 */
	private Driver real = null;


	/**
	 * The property name of the real JDBC driver full class name.
	 */
	private final static String DRIVER_CLASS_NAME_PROPERTY = "spy.driver";


	/**
	 * The property name of the SQL query to get the JDBC connection ID.
	 * 
	 * This query can be :
	 * <ul>
	 * <li><code>select @@spid</code> for Sybase</li>
	 * <li><code>select @@spid</code> for MS SQL-Server</li>
	 * <li><code>select @@session.identity</code> for MySQL</li>
	 * <li><code>select @@spidExec</code> for Oracle</li>
	 * </ul>
	 */
	private final static String CONNECTION_ID_SQL_PROPERTY = "spy.connection.id";


	/**
	 * The prefix of accepted JDBC url by the Spy JDBC driver when the real driver
	 * is no selected with the <code>spy.driver</code> system property. 
	 */
	private final static String URL_PREFIX = "jdbc:spy:";


	static {
		try {
			new SpyDriver();
		} catch ( Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 * Constructs a new Spy JDBC driver.
	 * @throws ClassNotFoundException When the real JDBC driver class was not found.
	 * @throws InstantiationException When the real JDBC driver cannot be instancied.
	 * @throws IllegalAccessException When cannot create a new instance of the real
	 * JDBC driver.
	 * @throws SQLException When cannot register the Spy JDBC driver or deregistrer
	 * the real JDBC driver.
	 */
	public SpyDriver () throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException {
		init();
	}


	/**
	 * Initializes the Spy JDBC driver.
	 * @throws ClassNotFoundException When the real JDBC driver class was not found.
	 * @throws InstantiationException When the real JDBC driver cannot be instancied.
	 * @throws IllegalAccessException When cannot create a new instance of the real
	 * JDBC driver.
	 * @throws SQLException When cannot register the Spy JDBC driver or deregistrer
	 * the real JDBC driver.
	 */
	private void init() throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException {
		DriverManager.registerDriver(this);
		registerRealDriver(System.getProperty(DRIVER_CLASS_NAME_PROPERTY));
	}


	/**
	 * Register the real JDBC driver in the Spy JDBC driver. Create a new instance
	 * of the real JDBC driver, store it and deregister this one from the driver
	 * manager. 
	 * @param className String The full name of the real JDBC driver.
	 * @throws ClassNotFoundException When the real JDBC driver class was not found.
	 * @throws InstantiationException When the real JDBC driver cannot be instancied.
	 * @throws IllegalAccessException When cannot create a new instance of the real
	 * JDBC driver.
	 * @throws SQLException When cannot deregistrer
	 * the real JDBC driver.
	 */
	private void registerRealDriver ( String className) throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException {
		if ( className!=null) {
			real = (Driver)Class.forName(className).newInstance();
			DriverManager.deregisterDriver(real);
		}
	}


	/**
	 * Extracts from a JDBC url the real driver full class name.
	 * @param url String The JDBC url from which extract the
	 * driver full class name.
	 * @return String The full class name otherwise <code>null</code>.
	 */
	private String getRealDriverClassName(String url) {
		String result = null;
		int index = url.indexOf(':',URL_PREFIX.length()+1);

		if ( index!=-1) {
			result = url.substring(URL_PREFIX.length(),index);
		}

		return result;
	}


	/**
	 * Returns the real JDBC url from the given url. Extracts the
	 * real url from a "spy url".
	 * @param url The JDBC url from which get the real url.
	 * @return String The real url otherwise the same.
	 */
	private String getRealUrl(String url) {
		return url!=null?url.startsWith(URL_PREFIX)?("jdbc"+url.substring(url.indexOf(':',URL_PREFIX.length()+1))):url:url;
	}


	/**
	 * Retrieves the real driver's major version number. Initially
	 * this should be <code>1</code>.
	 * @return int This driver's major version number.
	 * @see Driver#getMajorVersion()
	 */
	public int getMajorVersion() {
		return real!=null?real.getMajorVersion():1;
	}


	/**
	 * Gets the real driver's minor version number. Initially this
	 * should be <code>0</code>.
	 * @return int This driver's minor version number.
	 * @see Driver#getMinorVersion()
	 */
	public int getMinorVersion() {
		return real!=null?real.getMinorVersion():0;
	}


	/**
	 * Reports whether thz real driver is a genuine JDBC Compliant driver.
	 * @return boolean <code>true</code> if this driver is JDBC Compliant;
	 * <code>false</code> otherwise.
	 * @see Driver#jdbcCompliant()
	 */
	public boolean jdbcCompliant() {
		return real.jdbcCompliant();
	}


	/**
	 * Retrieves whether the real driver thinks that it can open a
	 * connection to the given URL.
	 * @param url String The URL of the database
	 * @return boolean <code>true</code> if this driver understands
	 * the given URL; <code>false</code> otherwise.
	 * @throws SQLException If a database access error occurs.
	 * @see Driver#acceptsURL(java.lang.String)
	 */
	public boolean acceptsURL(String url) throws SQLException {
		boolean result = false;


		if ( real==null && url.startsWith(URL_PREFIX)) {
			try {
				registerRealDriver(getRealDriverClassName(url));
			} catch ( IllegalAccessException e) {
				e.printStackTrace();
			} catch ( InstantiationException e) {
				e.printStackTrace();
			} catch ( ClassNotFoundException e) {
				e.printStackTrace();
			}
		}

		if ( real!=null) {
			result = real.acceptsURL(getRealUrl(url));
		}

		return result;
	}


	/**
	 * Attempts to make a database connection to the given URL.
	 * @param url String The URL of the database to which to connect.
	 * @param info Properties A list of arbitrary string tag/value
	 * pairs as connection arguments.
	 * @return Connection A Connection object that represents a
	 * connection to the URL.
	 * @throws SQLException If a database access error occurs.
	 * @see Driver#connect(java.lang.String, java.util.Properties)
	 */
	public Connection connect(String url, Properties info) throws SQLException {
		Connection con = null;


		if ( real==null && url.startsWith(URL_PREFIX)) {
			try {
				registerRealDriver(getRealDriverClassName(url));
			} catch ( IllegalAccessException e) {
				e.printStackTrace();
			} catch ( InstantiationException e) {
				e.printStackTrace();
			} catch ( ClassNotFoundException e) {
				e.printStackTrace();
			}
		}

		if ( real!=null) {
			con = new SpyConnection ( real.connect(getRealUrl(url),info),System.getProperty(CONNECTION_ID_SQL_PROPERTY));
		}

		return con;
	}


	/**
	 * Gets information about the possible properties for the real driver.
	 * @param url String The URL of the database to which to connect.
	 * @param info Properties A proposed list of tag/value pairs that will
	 * be sent on connect open.
	 * @return DriverPropertyInfo[] An array of DriverPropertyInfo objects
	 * describing possible properties.
	 * @throws SQLException If a database access error occurs.
	 * @see Driver#getPropertyInfo(java.lang.String, java.util.Properties)
	 */
	public DriverPropertyInfo[] getPropertyInfo(String url, Properties info) throws SQLException {
		return real!=null?real.getPropertyInfo(url,info):null;
	}
}
